package impl

import (
	"gitlab.okta-solutions.com/mashroom/backend/common/api/health"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/log"
	"gitlab.okta-solutions.com/mashroom/backend/maintenance/api"
	"gitlab.okta-solutions.com/mashroom/backend/maintenance/version"
	"google.golang.org/grpc"
	"net"
)

type Server interface {
	maintenance.MaintenanceServiceServer
	Serve(addr string)
	Background()
}

type serverImpl struct {
}

func (server *serverImpl) Background() {
	// background processes
}

func (server *serverImpl) Serve(addr string) {
	if listener, err := net.Listen("tcp", addr); err != nil {
		panic(err)
	} else {
		_ = log.SetHost("maintenance")
		grpcServer := grpc.NewServer()

		maintenance.RegisterMaintenanceServiceServer(grpcServer, server)

		healthServer := version.NewHealthServer()
		health.RegisterHealthServiceServer(grpcServer, healthServer)

		log.Infoln("maintenance started")
		if err := grpcServer.Serve(listener); err != nil {
			log.Errorln("gRPC error", err)
		}
	}
}

func NewServer() Server {
	server := &serverImpl{}
	go server.Background()
	return server
}
